/*
	BromScript - On Runtime Scripting Language.
    Copyright (C) 2012 - 2013  Alex Brouwer (Bromvlieg: http://brom.4o3.nl/)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "Executer.h"
#include "Variable.h"
#include "Function.h"
#include "Table.h"
#include "Converter.h"

#include <cstring>

using namespace Scratch;

namespace BromScript {
	bool CheckIfVarTrue(Variable* var){
		switch (var->Type) {
		case Variable::VariableType::Null: return false;
		case Variable::VariableType::Number: return *(double*)var->Value >= 0; break;
		case Variable::VariableType::String: return strlen(*(CString*)var->Value) > 0; break;
		case Variable::VariableType::Table: return ((Table*)var->Value)->Count > 0; break;
		case Variable::VariableType::Bool: return var->Value != null; break;
		default: return var->Value != null; break;
		}
	}

	Variable* Executer::New(ExecuteData* data) {
		bool oldcall = data->FuncCallback;
		data->FuncCallback = true;
		Variable* classobj = data->Function->InternalRun(data);
		data->FuncCallback = oldcall;

		if (classobj->Type == Variable::VariableType::Null){
			data->BromScript->Error("Cannot create an instance of null");
			return data->BromScript->GC.RegisterVariable();
		}

		if (classobj->Type != Variable::VariableType::Class){
			data->BromScript->Error("Attempting to create a non class!");
			return data->BromScript->GC.RegisterVariable();
		}

		int params = data->Reader->ReadInt();
		List<Variable*> vars;
		for (int i = 0; i < params; i++){
			bool oldcall = data->FuncCallback;
			data->FuncCallback = true;
			vars.Add(data->Function->InternalRun(data));
			data->FuncCallback = oldcall;

			if (data->HasReturnValue || data->Function->ForceReturn)
				return data->Returning;
		}

		List<CString> setskeys;
		List<Variable*> setsvalues;
		int sets = data->Reader->ReadInt();
		for (int i = 0; i < sets; i++){
			setskeys.Add(data->Reader->ReadString());

			bool oldcall = data->FuncCallback;
			data->FuncCallback = true;
			setsvalues.Add(data->Function->InternalRun(data));
			data->FuncCallback = oldcall;

			if (data->HasReturnValue || data->Function->ForceReturn)
				return data->Returning;
		}

		Function* func = (Function*)classobj->Value;
		if (func->IsCpp){
			Userdata* ud = null;
			for (int i = 0; i < data->BromScript->RegisteredUserdataTypes.Count; i++){
				if (data->BromScript->RegisteredUserdataTypes[i]->Name == func->Name){
					ud = data->BromScript->RegisteredUserdataTypes[i];
					break;
				}
			}

			if (ud->Ctor == null){
				data->BromScript->Error("Cannot create type, no constructor found!");
				delete func;
				return data->BromScript->GC.RegisterVariable();
			}

			Userdata* nud = ud->Copy();

			Variable* ret = data->BromScript->GC.RegisterVariable();
			ret->Type = (Variable::VariableType)nud->TypeID;
			ret->Value = nud;

			ArgumentData* args = new ArgumentData();
			args->BromScript = data->BromScript;
			args->SetVariableData(&vars);
			nud->TypeData = ud->Ctor(data->BromScript, args);
			nud->Dtor = ud->Dtor;
			nud->CallDTor = true;

			for (int i = 0; i < sets; i++){
				bool found = false;
				for (int i2 = 0; i2 < nud->Members.Count; i2++){
					if (nud->Members[i2]->Name == setskeys[i]){
						Variable* tmpvar = new Variable();
						tmpvar->Type = (Variable::VariableType)nud->Members[i2]->TypeID;
						tmpvar->Value = nud->Members[i2]->Copy();
						((Userdata*)tmpvar->Value)->TypeData = (char*)nud->TypeData + nud->Members[i2]->Offset;
						Converter::SetMember(tmpvar, setsvalues[i], setskeys[i]);
						delete tmpvar;

						found = true;
						break;
					}
				}

				if (!found){
					delete args;
					data->BromScript->Error(CString("Cannot find index ") + setskeys[i] + " on object type " + nud->Name);
					return ret;
				}
			}

			delete args;
			return ret;

		}else{
			// AWNFAWIFIUWFWAHFWAHU
		}

		return null;
	}

	void Executer::CreateFunction(ExecuteData* data) {
		CString funcname = data->Reader->ReadString();
		CString filename = data->Reader->ReadString();
		int codelen = data->Reader->ReadInt();

		Function* func = new Function(data->BromScript);
		func->Name = funcname;
		func->Filename = filename;
		func->Parent = data->Function;
		func->Code = data->Reader->ReadBytes(codelen);

		Variable* var = data->BromScript->GC.RegisterVariable();
		var->Value = func;
		var->Type = Variable::VariableType::Function;

		List<CString>* params = data->Reader->ReadStrings();
		for (int i = 0; i < params->Count; i++)
			func->Parameters.Add(CString(params->Get(i)));

		delete params;

		List<CString>* keys = data->Reader->ReadStrings();

		bool islocal = data->Reader->ReadBool();
		if (keys->Count == 1){
			data->Function->SetVar(funcname, var, !islocal);
		}else{
			Variable* curvar = data->Function->GetVar(funcname);
			Variable* prevvar = null;

			if (curvar->Type == Variable::VariableType::Null){
				data->BromScript->Error("Cannot index a null value");
				delete keys;
				return;
			}

			if (curvar->Type != Variable::VariableType::Table){
				data->BromScript->Error("Cannot index a non table value");
				delete keys;
				return;
			}

			for (int i = 1; i < keys->Count; i++){
				if (i == keys->Count - 1){
					if (curvar->Type == Variable::VariableType::Null){
						data->BromScript->Error("Cannot index a null value");
						delete keys;
						return;
					}

					if (curvar->Type != Variable::VariableType::Table && curvar->Type <= Variable::VariableType::Userdata){
						data->BromScript->Error("Cannot index a non table value");
						delete keys;
						return;
					}

					CString endkey = keys->Get(keys->Count - 1);
					if (curvar->Type == Variable::VariableType::Table){
						Table* tbl = (Table*)curvar->Value;
						tbl->Set(endkey, var);
						delete keys;
						return;
					}else{
						bool found = false;
						Userdata* ud = (Userdata*)prevvar->Value;
						for (int i = 0; i < ud->Members.Count; i++){
							if (ud->Members[i]->Name == endkey){
								curvar->Value = ud->Members[i]->Copy();
								((Userdata*)(curvar->Value))->TypeData = (byte*)ud->TypeData + ud->Members[i]->Offset;
								curvar->Type = (Variable::VariableType)ud->Members[i]->TypeID;
								curvar->IsCpp = true;
								curvar->CppData;
								found = true;

								if (!Converter::SetMember(curvar, var, endkey)){
									data->BromScript->Error("Failed to set member");
									delete keys;
									return;
								}

								delete keys;
								return;
							}
						}

						data->BromScript->Error(CString("Unknown userdata index '") + endkey + "'");
					}

					delete keys;
					return;
				}

				if (curvar->Type == Variable::VariableType::Table){
					Table* CurrentTable = (Table*)curvar->Value;

					prevvar = curvar;
					curvar = CurrentTable->Get(keys->Get(i));
				}else if (curvar->Type > Variable::VariableType::Userdata){
					Userdata* ud = (Userdata*)curvar->Value;

					prevvar = curvar;
					curvar = data->BromScript->GC.RegisterVariable();

					bool found = false;
					for (int i = 0; i < ud->Members.Count; i++){
						if (ud->Members[i]->Name == keys->Get(i)){
							curvar->Value = ud->Members[i]->Copy();
							((Userdata*)(curvar->Value))->TypeData = (byte*)ud->TypeData + ud->Members[i]->Offset;
							curvar->Type = (Variable::VariableType)ud->Members[i]->TypeID;
							curvar->IsCpp = true;
							curvar->CppData;
							found = true;
							break;
						}
					}

					if (!found){
						for (int i2 = 0; i2 < ud->Functions.Count(); i2++){
							if (ud->Functions.GetKeyByIndex(i2) == keys->Get(i)){
								Function* func = new Function(data->BromScript);
								func->CppFunc = ud->Functions.GetValueByIndex(i2);
								func->IsCpp = true;
								func->Name = keys->Get(i);
								func->Filename = "C++";

								curvar->Type = Variable::VariableType::Function;
								curvar->Value = func;

								found = true;
								break;
							}
						}

						if (!found){
							data->BromScript->Error(CString("Unknown userdata index '") + keys->Get(i) + "'");
							delete keys;
							return;
						}
					}
				}else{
					data->BromScript->Error("Cannot index a non table value");
					delete keys;
					return;
				}
			}
		}

		delete keys;
	}

	void Executer::SetTableIndex(ExecuteData* data){
		data->Reader->ReadByte(); // ignore, we should remove this in our compiler.

		CString tblname = data->Reader->ReadString();
		CString fullvarname = tblname;
		int loops = data->Reader->ReadByte();

		Variable* prevvar = null;
		Variable* curvar = data->Function->GetVar(tblname);

		CString key;

		for (int i = 0; i <= loops; i++){
			bool oldcall = data->FuncCallback;
			data->FuncCallback = true;
			Variable* keyvar = data->Function->InternalRun(data);
			data->FuncCallback = oldcall;

			if (data->HasReturnValue || data->Function->ForceReturn)
				return;

			if (i == loops){
				if (curvar->Type > Variable::VariableType::Userdata){
					if (!Converter::SetMember(curvar, keyvar, key)){
						data->BromScript->Error("Failed to set member");
						return;
					}
					return;
				}else{
					Table* tbl = (Table*)prevvar->Value;
					tbl->Set(key, keyvar);
					return;
				}
			}

			if (keyvar->Type == Variable::VariableType::Bool) key = keyvar->Value == null ? "False" : "True";
			else if (keyvar->Type == Variable::VariableType::Number) key = CString(*(double*)keyvar->Value);
			else if (keyvar->Type == Variable::VariableType::String) key = *(CString*)keyvar->Value;
			else{
				data->BromScript->Error(CString("Cannot index a table value using keytype ") + Converter::TypeToString(keyvar->Type));
				return;
			}

			if (curvar->Type == Variable::VariableType::Table){
				Table* CurrentTable = (Table*)curvar->Value;

				prevvar = curvar;
				curvar = CurrentTable->Get(key);
			}else if (curvar->Type > Variable::VariableType::Userdata){
				Userdata* ud = (Userdata*)curvar->Value;

				prevvar = curvar;
				curvar = data->BromScript->GC.RegisterVariable();

				bool found = false;
				for (int i = 0; i < ud->Members.Count; i++){
					if (ud->Members[i]->Name == key){
						curvar->Value = ud->Members[i]->Copy();
						((Userdata*)(curvar->Value))->TypeData = (byte*)ud->TypeData + ud->Members[i]->Offset;
						curvar->Type = (Variable::VariableType)ud->Members[i]->TypeID;
						curvar->IsCpp = true;
						curvar->CppData;
						found = true;
						break;
					}
				}

				if (!found){
					for (int i = 0; i < ud->Functions.Count(); i++){
						if (ud->Functions.GetKeyByIndex(i) == key){
							Function* func = new Function(data->BromScript);
							func->CppFunc = ud->Functions.GetValueByIndex(i);
							func->IsCpp = true;
							func->Name = key;
							func->Filename = "C++";

							curvar->Type = Variable::VariableType::Function;
							curvar->Value = func;

							found = true;
							break;
						}
					}

					if (!found){
						data->BromScript->Error(CString("Unknown userdata index '") + key + "'");
						return;
					}
				}
			}else{
				data->BromScript->Error("Cannot index a non table value");
				return;
			}

			fullvarname += CString("[") + key + "]";
		}
	}

	Variable* Executer::GetTableIndex(ExecuteData* data){
		data->Reader->ReadByte(); // ignore, we should remove this in our compiler.

		CString tblname = data->Reader->ReadString();
		CString fullvarname = tblname;
		int loops = data->Reader->ReadByte();

		Variable* prevvar = null;
		Variable* curvar = data->Function->GetVar(tblname);

		for (int i = 0; i < loops; i++){
			bool oldcall = data->FuncCallback;
			data->FuncCallback = true;
			Variable* keyvar = data->Function->InternalRun(data);
			data->FuncCallback = oldcall;

			if (data->HasReturnValue || data->Function->ForceReturn)
				return data->BromScript->GC.RegisterVariable();

			CString key;
			if (keyvar->Type == Variable::VariableType::Bool) key = keyvar->Value == null ? "False" : "True";
			else if (keyvar->Type == Variable::VariableType::Number) key = CString(*(double*)keyvar->Value);
			else if (keyvar->Type == Variable::VariableType::String) key = *(CString*)keyvar->Value;
			else{
				data->BromScript->Error(CString("Cannot index a table value using keytype ") + Converter::TypeToString(keyvar->Type));
				return data->BromScript->GC.RegisterVariable();
			}

			if (curvar->Type == Variable::VariableType::Table){
				Table* CurrentTable = (Table*)curvar->Value;

				prevvar = curvar;
				curvar = CurrentTable->Get(key);
			}else if (curvar->Type > Variable::VariableType::Userdata){
				Userdata* ud = (Userdata*)curvar->Value;

				prevvar = curvar;
				curvar = data->BromScript->GC.RegisterVariable();

				bool found = false;
				for (int i = 0; i < ud->Members.Count; i++){
					if (ud->Members[i]->Name == key){
						curvar->Value = ud->Members[i]->Copy();
						((Userdata*)(curvar->Value))->TypeData = (byte*)ud->TypeData + ud->Members[i]->Offset;
						curvar->Type = (Variable::VariableType)ud->Members[i]->TypeID;
						curvar->IsCpp = true;
						curvar->CppData = ud;
						found = true;
						break;
					}
				}

				if (!found){
					for (int i = 0; i < ud->Functions.Count(); i++){
						if (ud->Functions.GetKeyByIndex(i) == key){
							Function* func = new Function(data->BromScript);
							func->CppFunc = ud->Functions.GetValueByIndex(i);
							func->IsCpp = true;
							func->Name = key;
							func->Filename = "C++";

							curvar->Type = Variable::VariableType::Function;
							curvar->Value = func;

							found = true;
							break;
						}
					}

					if (!found){
						data->BromScript->Error(CString("Unknown userdata index '") + key + "'");
						return data->BromScript->GC.RegisterVariable();
					}
				}
			}else{
				data->BromScript->Error("Cannot index a non table value");
				return data->BromScript->GC.RegisterVariable();
			}

			fullvarname += CString("[") + key + "]";
		}


		bool exec = data->Reader->ReadBool();
		if (exec){
			if (curvar->Type != Variable::VariableType::Function){
				data->BromScript->Error(CString("Cannot execute a non function value, varname: ") + fullvarname);
				return data->BromScript->GC.RegisterVariable();
			}

			int params = data->Reader->ReadInt();
			List<Variable*> vars;

			for (int i = 0; i < params; i++){
				bool oldcall = data->FuncCallback;
				data->FuncCallback = true;
				vars.Add(data->Function->InternalRun(data));
				data->FuncCallback = oldcall;

				if (data->HasReturnValue || data->Function->ForceReturn)
					return data->Returning;
			}

			Function* func = (Function*)curvar->Value;
			curvar = func->Run(&vars, prevvar);
		}

		return curvar;
	}

	void Executer::SetVar(ExecuteData* data){
		CString key = data->Reader->ReadString();

		bool oldcall = data->FuncCallback;
		data->FuncCallback = true;
		Variable* var = data->Function->InternalRun(data);
		data->FuncCallback = oldcall;

		if (data->HasReturnValue || data->Function->ForceReturn)
			return;

		data->Function->SetVar(key, var, true);
	}

	void Executer::LSetVar(ExecuteData* data){
		CString key = data->Reader->ReadString();

		bool oldcall = data->FuncCallback;
		data->FuncCallback = true;
		Variable* var = data->Function->InternalRun(data);
		data->FuncCallback = oldcall;

		if (data->HasReturnValue || data->Function->ForceReturn)
			return;

		data->Function->SetVar(key, var, false);
	}

	Variable* Executer::GetVar(ExecuteData* data){
		CString key = data->Reader->ReadString();
		Variable* var = data->Function->GetVar(key);

		return var;
	}

	void Executer::If(ExecuteData* data){
		byte skipnextifs = data->Reader->ReadByte();
		int ifstatmentsize = data->Reader->ReadInt(); // unneeded, remove?

		bool oldcall = data->FuncCallback;
		data->FuncCallback = true;
		Variable* var = data->Function->InternalRun(data);
		data->FuncCallback = oldcall;

		if (data->HasReturnValue || data->Breaking || data->Continueing || data->Function->ForceReturn)
			return;

		int codelen = data->Reader->ReadInt();

		if (!CheckIfVarTrue(var)){
			data->Reader->Pos += codelen;
			return;
		}

		int oldpos = data->Reader->Pos;

		oldcall = data->FuncCallback;
		data->FuncCallback = false;
		data->Function->InternalRun(data);
		data->FuncCallback = oldcall;
		if (data->HasReturnValue || data->Breaking || data->Continueing || data->Function->ForceReturn)
			return;

		// incase it returns in the middle, would be bad if we enter at a random point in the code :V
		data->Reader->Pos = oldpos + codelen;
		while(skipnextifs == 1){
			data->Reader->Pos++;
			skipnextifs = data->Reader->ReadByte();

			// skip code data for elseifs
			data->Reader->Pos += data->Reader->ReadInt();
			data->Reader->Pos += data->Reader->ReadInt();
		}

		// only one else..... well lets hope, cus it's only possibru once!
		if (skipnextifs == 2){
			// skip code
			data->Reader->ReadByte();
			data->Reader->Pos += data->Reader->ReadInt();
		}
	}

	void Executer::While(ExecuteData* data){
		int checklen = data->Reader->ReadInt();
		int checkpos = data->Reader->Pos;

		while(true){
			bool oldcall = data->FuncCallback;
			data->FuncCallback = true;
			Variable* var = data->Function->InternalRun(data);
			data->FuncCallback = oldcall;

			if (data->HasReturnValue || data->Breaking || data->Continueing || data->Function->ForceReturn)
				return;

			data->Reader->Pos = checkpos + checklen;

			int codelen = data->Reader->ReadInt();
			if (!CheckIfVarTrue(var)){
				data->Reader->Pos = checkpos + checklen + codelen + 5; // +5, End byte and the int len for the bodycode, we should move this up.
				return;
			}

			oldcall = data->FuncCallback;
			data->FuncCallback = false;
			data->Function->InternalRun(data);
			data->FuncCallback = oldcall;

			if (data->HasReturnValue || data->Function->ForceReturn)
				return;

			if (data->Breaking){
				data->Breaking = false;
				data->Reader->Pos = checkpos + checklen + codelen + 5; // +5, End byte and the int len for the bodycode, we should move this up.
				return;
			}

			data->Continueing = false;
			data->Reader->Pos = checkpos;// goto if statment
		}
	}

	void Executer::For(ExecuteData* data){
		int initlen = data->Reader->ReadInt();
		int iflen = data->Reader->ReadInt();
		int blocksize = data->Reader->ReadInt();
		int afterlen = data->Reader->ReadInt();

		bool oldcall = data->FuncCallback;
		data->FuncCallback = false;
		data->Function->InternalRun(data);
		data->FuncCallback = oldcall;

		int startpos = data->Reader->Pos;
		while(true){
			bool oldcall = data->FuncCallback;
			data->FuncCallback = true;
			Variable* var = data->Function->InternalRun(data);
			data->FuncCallback = oldcall;

			data->Reader->Pos = startpos + iflen;

			if (data->HasReturnValue || data->Function->ForceReturn)
				return;

			if (!CheckIfVarTrue(var)){
				data->Reader->Pos = startpos + iflen + blocksize + afterlen;
				return;
			}

			oldcall = data->FuncCallback;
			data->FuncCallback = false;
			data->Function->InternalRun(data);
			data->FuncCallback = oldcall;

			data->Reader->Pos = startpos + iflen + blocksize;

			if (data->HasReturnValue || data->Function->ForceReturn)
				return;

			if (data->Breaking){
				data->Breaking = false;
				data->Reader->Pos = startpos + iflen + blocksize + afterlen;
				return;
			}

			oldcall = data->FuncCallback;
			data->FuncCallback = false;
			data->Function->InternalRun(data);
			data->FuncCallback = oldcall;

			data->Reader->Pos = startpos + iflen + blocksize + afterlen;

			if (data->HasReturnValue || data->Function->ForceReturn)
				return;

			data->Continueing = false;
			data->Reader->Pos = startpos; // back to the depths we go!
		}
	}

	void Executer::ForEach(ExecuteData* data){
		CString key = data->Reader->ReadString();
		CString value = data->Reader->ReadString();

		bool oldcall = data->FuncCallback;
		data->FuncCallback = true;
		Variable* var = data->Function->InternalRun(data);
		data->FuncCallback = oldcall;

		if (data->HasReturnValue || data->Function->ForceReturn)
			return;

		if (var->Type == Variable::VariableType::Null){
			data->BromScript->Error(CString("Cannot foreach a null value! foreach (") + key + ", " + value + ", null)");
			return;
		}

		if (var->Type != Variable::VariableType::Table){
			data->BromScript->Error(CString("Cannot foreach a non Table value! foreach (") + key + ", " + value + ", null)");
			return;
		}

		Table* tbl = (Table*)var->Value;

		int codelen = data->Reader->ReadInt();
		int codepos = data->Reader->Pos;
		for (int i = 0; i < tbl->Count; i++){
			Variable* valvar = tbl->GetByIndex(i);

			Variable* keyvar = data->BromScript->GC.RegisterVariable();
			keyvar->Type = Variable::VariableType::String;
			keyvar->Value = Converter::StringToPointer(tbl->GetKeyByIndex(i));

			data->Function->SetVar(key, keyvar, false);
			data->Function->SetVar(value, valvar, false);

			bool oldcall = data->FuncCallback;
			data->FuncCallback = false;
			data->Function->InternalRun(data);
			data->FuncCallback = oldcall;

			if (data->HasReturnValue || data->Function->ForceReturn)
				return;

			if (data->Breaking){
				data->Breaking = false;
				data->Reader->Pos = codepos + codelen;
				return;
			}

			data->Continueing = false;
			data->Reader->Pos = codepos; // back to the depths we go!
		}

		data->Reader->Pos = codepos + codelen;
	}

	void Executer::Else(ExecuteData* data){
		// not realy needed anymore i guess. could be removed.
		int codelen = data->Reader->ReadInt();

		bool oldcall = data->FuncCallback;
		data->FuncCallback = false;
		data->Function->InternalRun(data);
		data->FuncCallback = oldcall;
	}

	bool ShouldDoArithmatic(int nf){
#pragma message("WHEN FIXING MODULO ENUM, THIS SHOULD BE CHANGED!!!")

		return
			(nf >= (int)Misc::ArithmaticFuncs::Add && nf <= (int)Misc::ArithmaticFuncs::Multiply) ||
			(nf >= (int)Misc::ArithmaticFuncs::And && nf <= (int)Misc::ArithmaticFuncs::NotEqual) ||
			nf == (int)Misc::ArithmaticFuncs::Modulo;
	}

	Variable* Executer::Merge(ExecuteData* data){
		int spos = data->Reader->Pos;
		int mergesize = data->Reader->ReadInt();

		bool oldcall = data->FuncCallback;
		data->FuncCallback = true;
		Variable* var = data->Function->InternalRun(data);
		data->FuncCallback = oldcall;
		
		if (data->HasReturnValue || data->Function->ForceReturn)
			return data->Returning;

		if (var->Type >= Variable::VariableType::Userdata)
			var = Converter::MemberToVariable(data->BromScript, var);

		Misc::ExecFuncs nf = (Misc::ExecFuncs)data->Function->Code[data->Reader->Pos];

		if (nf == Misc::ExecFuncs::MergeEnd){
			data->Reader->Pos++;
			return var;
		}

		while(ShouldDoArithmatic((int)nf)){
			data->Reader->Pos++;

			// dont resume when left side is false at AND op
			if (nf == Misc::ArithmaticFuncs::And && !CheckIfVarTrue(var)){
				Variable* ret = data->BromScript->GC.RegisterVariable();
				ret->Type = Variable::VariableType::Bool;
				ret->Value = null;
				data->Reader->Pos = spos + mergesize - 1;
				return ret;
			}

			// dont resume when left side is true at OR op
			if (nf == Misc::ArithmaticFuncs::Or && CheckIfVarTrue(var)){
				Variable* ret = data->BromScript->GC.RegisterVariable();
				ret->Type = Variable::VariableType::Bool;
				ret->Value = (void*)1;
				data->Reader->Pos = spos + mergesize - 1;
				return ret;
			}

			oldcall = data->FuncCallback;
			data->FuncCallback = true;
			Variable* var2 = data->Function->InternalRun(data);
			data->FuncCallback = oldcall;

			if (data->HasReturnValue || data->Breaking || data->Continueing || data->Function->ForceReturn)
				return data->Returning;

			if (var2->Type >= Variable::VariableType::Userdata)
				var2 = Converter::MemberToVariable(data->BromScript, var2);


			var = Executer::Arithmatic(data, var, var2, (Misc::ArithmaticFuncs)nf);

			nf = (Misc::ExecFuncs)data->Function->Code[data->Reader->Pos];
		}

		if (nf == Misc::ExecFuncs::MergeEnd){
			data->Reader->Pos++;
			return var;
		}

		return var;
	}

	Variable* Executer::ExecuteFunction(ExecuteData* data) {
		CString name = data->Reader->ReadString();
		int params = data->Reader->ReadInt();

		List<Variable*> vars;
		for (int i = 0; i < params; i++){
			bool oldcall = data->FuncCallback;
			data->FuncCallback = true;
			vars.Add(data->Function->InternalRun(data));
			data->FuncCallback = oldcall;

			if (data->HasReturnValue || data->Function->ForceReturn)
				return data->Returning;
		}

		Variable* var = data->Function->GetVar(name);

		if (var == null){
			data->BromScript->Error("Trying to execute a null value!");
			return data->BromScript->GC.RegisterVariable();
		}

		if (var->Type != Variable::VariableType::Function){
			data->BromScript->Error(CString("Trying to execute a non function! Varname: ") + name);
			return data->BromScript->GC.RegisterVariable();
		}

		Function* func = (Function*)var->Value;
		Variable* ret = func->Run(&vars);
		return ret;
	}

	Variable* Executer::Return(ExecuteData* data) {
		bool oldcall = data->FuncCallback;
		data->FuncCallback = true;
		data->Returning = data->Function->InternalRun(data);
		data->FuncCallback = oldcall;

		data->HasReturnValue = true;
		return data->Returning;
	}

	Variable* Executer::GetBool(ExecuteData* data) {
		Variable* var = data->BromScript->GC.RegisterVariable();
		var->Value = Converter::BoolToPointer(data->Reader->ReadBool());
		var->Type = Variable::VariableType::Bool;
		return var;
	}

	Variable* Executer::GetString(ExecuteData* data) {
		Variable* var = data->BromScript->GC.RegisterVariable();
		var->Value = new CString(data->Reader->ReadString());
		var->Type = Variable::VariableType::String;
		return var;
	}

	Variable* Executer::GetNumber(ExecuteData* data) {
		Variable* var = data->BromScript->GC.RegisterVariable();
		var->Value = Converter::NumberToPointer(data->Reader->ReadDouble());
		var->Type = Variable::VariableType::Number;
		return var;
	}

	Variable* Executer::GetTable(ExecuteData* data) {
		int varcount = data->Reader->ReadInt();

		Variable* var = data->BromScript->GC.RegisterVariable();
		Table* tbl = new Table(data->BromScript);
		var->Value = tbl;
		var->Type = Variable::VariableType::Table;

		for (int i = 0; i < varcount; i++){
			bool oldcall = data->FuncCallback;
			data->FuncCallback = true;
			Variable* ret = data->Function->InternalRun(data);
			data->FuncCallback = oldcall;

			tbl->Set(CString((double)i), ret);
		}

		return var;
	}

	void Arithmatic_DivideNumber(Variable* ret, Variable* a, Variable* b){
		ret->Type = Variable::VariableType::Number;
		ret->Value = Converter::NumberToPointer(*(double*)a->Value / *(double*)b->Value);
	}

	void Arithmatic_MultiplyNumber(Variable* ret, Variable* a, Variable* b){
		ret->Type = Variable::VariableType::Number;
		ret->Value = Converter::NumberToPointer(*(double*)a->Value * *(double*)b->Value);
	}

	void Arithmatic_MultiplyString(Variable* ret, Variable* num, Variable* string){
		CString strtoadd(*(CString*)string->Value);
		CString str;
		int loops = (int)*(double*)num->Value;

		if (loops <= 0){
			ret->Type = Variable::VariableType::String;
			ret->Value = Converter::StringToPointer("");
			return;
		}

		for (int i = 0; i < loops; i++)
			str += strtoadd;

		if (*(double*)num->Value - loops > 0)
			str += strtoadd.Substring(0, (*(double*)num->Value - loops) * str.Size());

		ret->Type = Variable::VariableType::String;
		ret->Value = Converter::StringToPointer(str);
	}

	void Arithmatic_SubstractNumber(Variable* ret, Variable* a, Variable* b){
		ret->Type = Variable::VariableType::Number;
		ret->Value = Converter::NumberToPointer(*(double*)a->Value - *(double*)b->Value);
	}

	void Arithmatic_DivideString(Instance* bs, Variable* ret, Variable* a, Variable* b){
		CString str = *(CString*)a->Value;
		Table* tbl = new Table(bs);

		int strlen = str.Size();
		int partlen = strlen / *(double*)b->Value;
		
		ret->Type = Variable::VariableType::Table;
		ret->Value = tbl;

		if (partlen <= 0)
			return;
		
		for (double i = 0; i < strlen; i += partlen)
			tbl->Set(CString(i), Converter::ToVariable(str.Substring(i, strlen - i < partlen ? strlen - i : partlen)));
	}

	void Arithmatic_AddNumber(Variable* ret, Variable* a, Variable* b){
		ret->Type = Variable::VariableType::Number;
		ret->Value = Converter::NumberToPointer(*(double*)a->Value + *(double*)b->Value);
	}

	void Arithmatic_AddString(Variable* ret, Variable* a, Variable* b){
		ret->Type = Variable::VariableType::String;
		ret->Value = Converter::StringToPointer(*(CString*)a->Value + *(CString*)b->Value);
	}
	
	void Arithmatic_AddNumberToString(Variable* ret, Variable* a, Variable* b){
		ret->Type = Variable::VariableType::String;
		ret->Value = Converter::StringToPointer(*(CString*)b->Value + CString(*(double*)a->Value));
	}

	void Arithmatic_AddNumberToString2(Variable* ret, Variable* a, Variable* b){
		ret->Type = Variable::VariableType::String;
		ret->Value = Converter::StringToPointer(CString(*(double*)a->Value) + *(CString*)b->Value);
	}

	Variable* Executer::Arithmatic(ExecuteData* data, Variable* a, Variable* b, Misc::ArithmaticFuncs af){
		Variable* ret = data->BromScript->GC.RegisterVariable();

		switch(af){
		case Misc::ArithmaticFuncs::Add:
			if (a->Type == Variable::VariableType::Number){
				if (b->Type == Variable::VariableType::Number) Arithmatic_AddNumber(ret, a, b);
				if (b->Type == Variable::VariableType::String) Arithmatic_AddNumberToString2(ret, a, b);
			}else if (a->Type == Variable::VariableType::String){
				if (b->Type == Variable::VariableType::String) Arithmatic_AddString(ret, a, b);
				if (b->Type == Variable::VariableType::Number) Arithmatic_AddNumberToString(ret, b, a);
			}
			break;

		case Misc::ArithmaticFuncs::Substract:
			if (a->Type == Variable::VariableType::Number){
				if (b->Type == Variable::VariableType::Number) Arithmatic_SubstractNumber(ret, a, b);
			}
			break;

		case Misc::ArithmaticFuncs::Divide:
			if (a->Type == Variable::VariableType::Number){
				if (b->Type == Variable::VariableType::Number) Arithmatic_DivideNumber(ret, a, b);
			}else if (a->Type == Variable::VariableType::String){
				if (b->Type == Variable::VariableType::Number) Arithmatic_DivideString(data->BromScript, ret, a, b);
			}
			break;

		case Misc::ArithmaticFuncs::Multiply:
			if (a->Type == Variable::VariableType::Number){
				if (b->Type == Variable::VariableType::Number) Arithmatic_MultiplyNumber(ret, a, b);
				else if (b->Type == Variable::VariableType::String) Arithmatic_MultiplyString(ret, a, b);
			}else if (a->Type == Variable::VariableType::String){
				if (b->Type == Variable::VariableType::Number) Arithmatic_MultiplyString(ret, b, a); // rotated a and b, usage num and string
			}
			break;

		case Misc::ArithmaticFuncs::NotEqual: // we reverse this later on, saves code
		case Misc::ArithmaticFuncs::Equal:
			ret->Type = Variable::VariableType::Bool;
			if (a->Type != b->Type) ret->Value = Converter::BoolToPointer(false);
			else if (a->Type == Variable::VariableType::Bool) ret->Value = Converter::BoolToPointer(a->Value == b->Value);
			else if (a->Type == Variable::VariableType::Number) ret->Value = Converter::BoolToPointer(*(double*)a->Value == *(double*)b->Value);
			else if (a->Type == Variable::VariableType::String) ret->Value = Converter::BoolToPointer(*(CString*)a->Value == *(CString*)b->Value);
			else ret->Value = Converter::BoolToPointer(a->Value == b->Value);
			break;

		case Misc::ArithmaticFuncs::GreaterThan:
			ret->Type = Variable::VariableType::Bool;
			if (a->Type != b->Type || a->Type != Variable::VariableType::Number){
				ret->Value = Converter::BoolToPointer(false);
			}else{
				ret->Value = Converter::BoolToPointer(*(double*)a->Value > *(double*)b->Value);
			}
			break;

		case Misc::ArithmaticFuncs::GreaterOrEqual:
			ret->Type = Variable::VariableType::Bool;
			if (a->Type != b->Type || a->Type != Variable::VariableType::Number){
				ret->Value = Converter::BoolToPointer(false);
			}else{
				ret->Value = Converter::BoolToPointer(*(double*)a->Value >= *(double*)b->Value);
			}
			break;

		case Misc::ArithmaticFuncs::LessThan:
			ret->Type = Variable::VariableType::Bool;
			if (a->Type != b->Type || a->Type != Variable::VariableType::Number){
				ret->Value = Converter::BoolToPointer(false);
			}else{
				ret->Value = Converter::BoolToPointer(*(double*)a->Value < *(double*)b->Value);
			}
			break;

		case Misc::ArithmaticFuncs::LessOrEqual:
			ret->Type = Variable::VariableType::Bool;
			if (a->Type != b->Type || a->Type != Variable::VariableType::Number){
				ret->Value = Converter::BoolToPointer(false);
			}else{
				ret->Value = Converter::BoolToPointer(*(double*)a->Value <= *(double*)b->Value);
			}
			break;
			
		case Misc::ArithmaticFuncs::Modulo:
			if (a->Type != Variable::VariableType::Number || b->Type != Variable::VariableType::Number)
				break;

			ret->Type = Variable::VariableType::Number;
			ret->Value = Converter::NumberToPointer(fmod((double)*(double*)a->Value, *(double*)b->Value));
			break;
			
		case Misc::ArithmaticFuncs::And:
			ret->Type = Variable::VariableType::Bool;
			ret->Value = Converter::BoolToPointer(CheckIfVarTrue(a) && CheckIfVarTrue(b));
			break;

		case Misc::ArithmaticFuncs::Or:
			ret->Type = Variable::VariableType::Bool;
			ret->Value = Converter::BoolToPointer(CheckIfVarTrue(a) || CheckIfVarTrue(b));
			break;
		}

		if (af == Misc::ArithmaticFuncs::NotEqual) // just rotate around equal, less code, same outcome.
			ret->Value = Converter::BoolToPointer(ret->Value == null);

		if (ret->Type == Variable::VariableType::Null)
			data->BromScript->Error(CString("Invalid arithmatic function!, trying to do ") + Converter::TypeToString(data->BromScript, a->Type) + " " + Converter::ArithmaticToString(af) + " " + Converter::TypeToString(data->BromScript, b->Type));

		return ret;
	}
}