/*
	BromScript - On Runtime Scripting Language.
    Copyright (C) 2012 - 2013  Alex Brouwer (Bromvlieg: http://brom.4o3.nl/)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef BROMSCRIPT_SIF_INCLUDED
#define BROMSCRIPT_SIF_INCLUDED

#ifndef SCRATCH_NO_GLOBFUNC
#define SCRATCH_NO_GLOBFUNC
#endif

#include "Scratch/Scratch.h"
#include "List.h"

#ifndef BS_FUNCTION
#define BS_FUNCTION(fn) BromScript::Variable* fn(BromScript::Instance* bsi, BromScript::ArgumentData* args)
#define BS_FUNCTION_CTOR(fn) void* fn(BromScript::Instance* bsi, BromScript::ArgumentData* args)
#define BS_FUNCTION_DTOR(fn) void fn(BromScript::Instance* bsi, void* data)

#define BS_MODULE(fn) void fn(BromScript::Instance* bsi)
#define BS_ERRORFUNCTION(fn) void fn(BromScript::Instance* bsi, BromScript::CallStack* stack, int stacksize, const char* msg)

#define BS_USERDATA_GET(fn) BromScript::Variable* fn(void* ptr)
#define BS_USERDATA_SET(fn) void fn(void* ptr, BromScript::Variable* var)
#endif

#ifndef null
#define null 0
#endif

#ifndef BROMSCRIPT_CURRENTVERSION
#define BROMSCRIPT_CURRENTVERSION 1
#endif

#ifndef BS_PI
#define BS_PI 3.14159265359
#endif

namespace BromScript {
	using namespace Scratch;

	class ByteReader;
	class ByteWriter;
	class CallStack;
	class Function;
	class Instance;
	class ExecuteData;
	class Executer;
	class Variable;
	class ArgumentData;
	class GarbageCollector;
	class Table;
	class MemberData;
	class Userdata;
	class MemberType;
	class CompilerException;

	typedef unsigned char byte;
	typedef Variable* (*BSFunction)(Instance* bsi, ArgumentData* args);
	typedef void* (*BSFunctionCtor)(Instance* bsi, ArgumentData* args);
	typedef void (*BSFunctionDtor)(Instance* bsi, void* data);
	typedef void (*ErrorFunction)(Instance* bsi, CallStack* stack, int stacksize, const char* name);
	typedef void (*BSModuleFunction)(Instance*);
	
	typedef Variable* (*BSGetFunction)(void* ptr);
	typedef void (*BSSetFunction)(void* ptr, Variable* var);
}

#endif